@@ -155,11 +155,7 @@ h5tools_str_append(h5tools_str_t *str/*in,out*/, const char *fmt, ...)
          * to lack of buffer size, so try one more time after realloc more
          * buffer size before return NULL. 
          */
-        if (nchars < 0 
-#ifndef H5_VSNPRINTF_WORKS
-                && (HDstrlen(str->s) < str->nalloc)
-#endif
-                ) {
+        if (nchars < 0) {
             /* failure, such as bad format */
             return NULL;
         }
